package com.jpa.test;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;

import junit.framework.Assert;
import junit.framework.TestCase;

import com.jpa.Organisation;
import com.jpa.Person;

public class OrganisationTests extends TestCase {
	
    protected EntityManager em = TestUtils.EMF.createEntityManager();
    
    public OrganisationTests(String name) {
    	super(name);
    }
	
	@Override
	protected void setUp() throws Exception {
	    em.clear();
	}

	@Override
	protected void tearDown() throws Exception {
	}

	public void testOrganisationCreate() {
		EntityTransaction userTransaction = em.getTransaction();
	    userTransaction.begin();
	    Organisation organisation = TestUtils.createOrganisation();
	    em.persist(organisation);
	    userTransaction.commit();
	}
	
	public void testOrganisationFindByID() {
		// erstelle eine Organisation
		EntityTransaction userTransaction = em.getTransaction();
	    userTransaction.begin();
	    Organisation organisation = TestUtils.createOrganisation();
	    em.persist(organisation);
	    userTransaction.commit();
	    // merke die Inhalte:
	    long organisationID = organisation.getId();
	    String organisationName = organisation.getName();
	    // das große Vergessen:
	    em.clear();
	    
	    // finde Person erneut
	    Organisation organisationFound = em.find(Organisation.class, organisationID);
	    
	    Assert.assertEquals("Name der Organisation muss gleich sein.", organisationName, organisationFound.getName());
	}

	public void testOrganisationFindWithQuery() {
		// erstelle eine Organisation
		EntityTransaction userTransaction = em.getTransaction();
	    userTransaction.begin();
	    Organisation organisation = TestUtils.createOrganisation();
	    em.persist(organisation);
	    userTransaction.commit();
	    // merke die Inhalte:
	    long organisationID = organisation.getId();
	    String organisationName = organisation.getName();
	    // das große Vergessen:
	    em.clear();
	    
	    // finde Person erneut
	    Query query = em.createNamedQuery("Organisation.findByName");
	    query.setParameter("name", organisationName);
	    List<Organisation> organisations = query.getResultList();
	    
	    Assert.assertEquals("Nur eine Person dieses Namens erwartet", 1, organisations.size());
	    Organisation organisationFound = organisations.get(0);
	    
	    Assert.assertEquals("Name der Person muss gleich sein.", organisationID, organisationFound.getId());
	}
	
	public void testOrganisationWithPerson() {
		// erstelle eine Organisation
		EntityTransaction userTransaction = em.getTransaction();
	    userTransaction.begin();
	    Organisation organisation = TestUtils.createOrganisation();
	    Person person = TestUtils.createPerson();
	    person.addOrganisation(organisation); // setzt die Bidirektionalität korrekt!
	    em.persist(organisation);
	    userTransaction.commit();
	    
	    // merke die Inhalte:
	    long organisationID = organisation.getId();
	    String organisationName = organisation.getName();
	    String personName = person.getName();
	    // das große Vergessen:
	    em.clear();
	    
	    // finde Organisation erneut
	    Organisation organisationFound = em.find(Organisation.class, organisationID);

	    Assert.assertEquals("Name der Organisation muss gleich sein.", organisationName, organisationFound.getName());
	    // Lese nun Person:
	    System.out.println("!-------------------!");
	    Person personFound = organisationFound.getPerson();
	    Assert.assertEquals("Name der Personen muss gleich sein.", personName, personFound.getName());
	}

	public void testManyOrganisationWithPerson() {
		// erstelle viele Organisationen
		String orgName = "OE" + TestUtils.randomString();
		for (int i = 0; i < 10; i++) {
			EntityTransaction userTransaction = em.getTransaction();
			userTransaction.begin();
		    Organisation organisation = TestUtils.createOrganisation();
		    organisation.setName(orgName);
		    Person person = TestUtils.createPerson();
		    person.addOrganisation(organisation); // setzt die Bidirektionalität korrekt!
		    em.persist(organisation);
		    userTransaction.commit();
		    System.out.println("--Org created--");
		}
	    // das große Vergessen:
	    em.clear();
	    
	    // finde alle Organisationen wieder:
	    Query query = em.createNamedQuery("Organisation.findByName");
	    query.setParameter("name", orgName);
	    List<Organisation> organisations = query.getResultList();
	    System.out.println("--Orgs found--");
	    Assert.assertEquals("Es werden 10 Organisationen werwartet.", 10, organisations.size());
	    
	    // gehe durch alle Organisationen 
	    for (Organisation organisation : organisations) {
	    	Person person = organisation.getPerson();
	    	person.getName();
		    System.out.println("--Person read--");
		}
	}
	
	public void testManyOrganisationWithPersonJoinFetch() {
		// erstelle viele Organisationen
		String orgName = "OE" + TestUtils.randomString();
		for (int i = 0; i < 10; i++) {
			EntityTransaction userTransaction = em.getTransaction();
		    userTransaction.begin();
		    Organisation organisation = TestUtils.createOrganisation();
		    organisation.setName(orgName);
		    Person person = TestUtils.createPerson();
		    person.addOrganisation(organisation); // setzt die Bidirektionalität korrekt!
		    em.persist(organisation);
		    userTransaction.commit();
		}
	    // das große Vergessen:
	    em.clear();
	    
	    // finde alle Organisationen wieder:
	    Query query = em.createNamedQuery("Organisation.findByName.joinFetch");
	    query.setParameter("name", orgName);
	    List<Organisation> organisations = query.getResultList();
	    Assert.assertEquals("Es werden 10 Organisationen werwartet.", 10, organisations.size());
	    
	    // gehe durch alle Organisationen 
	    for (Organisation organisation : organisations) {
	    	Person person = organisation.getPerson();
	    	person.getName();
	    	System.out.println("--Person read--");
		}
	}

}